--########################################################################
--
--  Este arquivo contm as seguintes entidades :
--      ==> Decodificador para acesso  entrada/sada ou memria
--      ==> Memria na forma de um vetor, com o cdigo objeto em um package
--      ==> Display Interface
--
--   FERNANDO GEHM MORAES	03/agosto/2005
--   NEY LAERT VILAR CALAZANS	18/junho/2006
--
--########################################################################

--########################################################################
--
-- Decodificador para acesso  entrada/sada ou memria
--
-- Ney (18/06/2006): acrscimo de uma posio de memria para E/S,
-- 	disponibilizando a possibilidade de separao de entrada de dados
-- 	e entrada de sinais de controle (botes BTN1 a BTN5)
--########################################################################
library IEEE;
use IEEE.Std_Logic_1164.all;

entity es_decoder is
    port(ce, rw:  in std_logic;
         address: in std_logic_vector(7 downto 0);
         ceCPU, rwCPU, ceES, rwES: out std_logic);
end es_decoder;

architecture a1 of es_decoder is

  signal add_es  : std_logic;
  
begin
  
    -- 
    -- acessos aos endereos FC a FF designam acesso  entrada e sada
    --
    add_es <=	'1' when address > x"FB" else
		'0';
                
    ceCPU <=	'1' when ce='1' and add_es='0'  else '0';
    ceES  <=	'1' when ce='1' and add_es='1'  else '0';
    
    
    rwCPU <=	'1' when rw='1' and add_es='0'  else '0';
    rwES  <=	'1' when rw='1' and add_es='1'  else '0';
   
     
end a1;



--########################################################################
--
-- memria na forma de um vetor - cdigo objeto definido em package
--
--########################################################################
library IEEE;
use ieee.STD_LOGIC_UNSIGNED.all;
use ieee.std_logic_1164.all;
use work.object_code.all;                  -- cdigo objeto definido neste package

entity RAM_mem is
      port( clock, ce, rw : in  std_logic;
            address:  in  std_logic_vector(7 downto 0);
            datain:   in  std_logic_vector(7 downto 0);
            dataout:  out std_logic_vector(7 downto 0) );
end RAM_mem;

architecture RAM_mem of RAM_mem is 
begin    

   process(clock)
   begin
       if clock'event and clock='1' then
          if ce='1' and rw='0' then               
                  memoria(CONV_INTEGER(address)) <= datain;       -- writes in memory
          end if;  
          
          dataout <=  memoria(CONV_INTEGER(address)) ;              -- memory reading
          
       end if;
   end process;    
   
end RAM_mem;

--########################################################################
--
-- Display Interface
--
--########################################################################
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity dio4_dspl_drv is
        port (
                clock: in STD_LOGIC;
                reset: in STD_LOGIC;
                d4: in STD_LOGIC_VECTOR (6 downto 0);
                d3: in STD_LOGIC_VECTOR (6 downto 0);
                d2: in STD_LOGIC_VECTOR (6 downto 0);
                d1: in STD_LOGIC_VECTOR (6 downto 0);
                an: out STD_LOGIC_VECTOR (4 downto 1);
                dec_ddp: out STD_LOGIC_VECTOR (7 downto 0)
        );
end dio4_dspl_drv;


architecture dio4_dspl_drv of dio4_dspl_drv is
        signal ck_1KHz: std_logic;
        signal dig_selection: std_logic_vector (1 downto 0);
        signal selected_dig: std_logic_vector (5 downto 0);
begin
        -- 1KHz clock generation
        process (reset, clock)
        variable count_25K: integer range 0 to 25000;
        begin
                if reset='1' then
                        count_25K := 0;
                        ck_1KHz <= '0';
                elsif (clock'event and clock='1') then
                        count_25K := count_25K + 1;
                        if (count_25K = 24999) then
                                count_25K := 0;
                                ck_1KHz <= not ck_1KHz;
                        end if;
                end if;
        end process;
        
        -- 1KHz counter to select digit and register output
        process (reset, ck_1KHz)
        begin
                if reset='1' then
                        dig_selection <= (others => '0'); 
                        an <= (others => '1');                     -- Disable all displays
                elsif (ck_1KHz'event and ck_1KHz='1') then
                        -- a 2-bit Johnson counter              
                        dig_selection <= dig_selection(0)  & not dig_selection (1);
                        
                        if dig_selection="00" then
                            selected_dig <= d1(5 downto 0);
                            an <= "111"  & (not d1(6));
                        elsif dig_selection="01" then
                            selected_dig <= d2(5 downto 0);
                            an <= "11" & (not d2(6)) & "1";
                        elsif dig_selection="10" then
                            selected_dig <= d3(5 downto 0);
                            an <= "1"  & (not d3(6)) & "11";
                        else
                            selected_dig <= d4(5 downto 0);
                            an <= (not d4(6)) & "111";
                        end if;
                end if; 
        end process;
        
        -- digit 4-to-hex decoder
        with selected_dig (5 downto 1) select
        dec_ddp(7 downto 1) <=
                "0000001" when "00000", --0
                "1001111" when "00001", --1
                "0010010" when "00010", --2
                "0000110" when "00011", --3
                "1001100" when "00100", --4
                "0100100" when "00101", --5
                "0100000" when "00110", --6
                "0001111" when "00111", --7
                "0000000" when "01000", --8
                "0000100" when "01001", --9
                "0001000" when "01010", --A
                "1100000" when "01011", --b
                "0110001" when "01100", --C
                "1000010" when "01101", --d
                "0110000" when "01110", --E
                "0111000" when "01111", --F
                "0011000" when "10000", --P
                "1001000" when "10001", --H
                "1000001" when "10010", --U
                "1100011" when "10011", --u
                "1111111" when others; 
                
        -- and the decimal point
        dec_ddp(0) <= selected_dig(0);
                                                                                          
end dio4_dspl_drv;
